%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This scrip contains and generates input data to the Q-learning Algorithm 
% (QLearning.m) and to Life Cycle Simulator (LCSimulator.m). Most of the
% date is obtained from the excel file "Case.xlsx". The script calls upon
% the functions: "allcomb.m", "Fix_Information." and "Design_Compatibility."
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Obtain Variable Design State Parameters 
N_Var_Design = 6;   % Number of variable design state parameters 
N_ACC = 3; % Number of variable Accommodation (ACC) parameters 
N_MC = 3; % Number of variable Main Crane Capacity (MC) parameters 
N_LWI = 3; % Number of variable Light Well Intervention (LWI) parameters 
N_ROV = 2; % Number of variable Remotely Operated Vehicles (ROV) parameters 
N_PC = 2; % Number of variable Cabel laying equipment (PC) parameters 
N_MP = 2; % Number of variable Moonpool (MP) parameters 

%% Create Variable Design State Sets 
Set_ACC = 1:N_ACC;    % Set ACC
Set_MC = 1:N_MC;     % Set MC 
Set_LWI = 1:N_LWI;    % Set LWI
Set_ROV = 1:N_ROV;    % Set ROV
Set_PC = 1:N_PC;     % Set PC
Set_MP = 1:N_MP;     % Set MP

%% Fixed Design State Parameters
N_Fix_Design = 4;   % Number of fixed design state parameters
Length = 120;       % [m] 
Beam = 25;          % [m]
Depth = 10;         % [m]
DfC = [0 1 2]; % Degree of Changeability (DfC) [-]

%% Specify Starting State for Design
Start_Design = [2 2 1 2 1 1]; % Representing base design (I)
%Start_Design = [2 1 1 1 1 1]; % Representing base design (II)

%% General information 
Lifetime = 10; % Length [Years] of lifetime  considered 
N_Design = length(DfC); % Number of vessels analysed 
time_short = 1; % Length [Years] of short-term contract
time_long = 3; % Length [Years] of long-term contract
N_Space_Market = 6; % Number of possible market states to encounter
N_Space_Mission = 9; % Number of possible missions states to take
N_Strategy = 3; % Number of possible strategies
OpDays = 300; % Days in operation per year 
SDays = [70 30 20]; % Days to perform switch based on DfC level
disc = 0.08; % Select Dissocunt Rate (r)
gamma = (1/(1+disc)); % Discount factor 
% Probability of wining a contract
probwin = [0 0.2 0.8; 0 0.3 0.7; 0 0.4 0.6; 0 0.5 0.5; 0 0.6 0.4; 0 0.7 0.3];

%% Physical Design State Requirements
N_Requirement = 3; % Number of levels for technical requirements 
F_min = 1.5;      % Freeboard criteria [m]
GM_min = 0.15;    % GM criteria [m]
DA_min = 0;       % Deck area critaria [m^2]

%% Switching costs [mill. USD]
Switch_CM = cell(N_Var_Design,1);
Switch_CM{1} = xlsread('Case.xlsx',4,'D5:F7'); % ACC
Switch_CM{2} = xlsread('Case.xlsx',4,'D10:F12'); % MC 
Switch_CM{3} = xlsread('Case.xlsx',4,'D15:F17'); % LWI
Switch_CM{4} = xlsread('Case.xlsx',4,'D20:E21'); % ROV
Switch_CM{5} = xlsread('Case.xlsx',4,'D24:E25'); % PC
Switch_CM{6} = xlsread('Case.xlsx',4,'D28:E29'); % MP

%% Mission Requirements to each variable design parameter:
% (ACC, MC, LWI, ROV, PC, MP)
Requirement_Matrix = cell(N_Space_Mission,1);
Requirement_Matrix{1} = xlsread('Case.xlsx',2,'D5:I7'); % OSC
Requirement_Matrix{2} = xlsread('Case.xlsx',2,'D8:I10'); % IMR
Requirement_Matrix{3} = xlsread('Case.xlsx',2,'D11:I13'); % LWI
Requirement_Matrix{4} = xlsread('Case.xlsx',2,'D14:I16'); % ODS
Requirement_Matrix{5} = xlsread('Case.xlsx',2,'D17:I19'); % ACC
Requirement_Matrix{6} = xlsread('Case.xlsx',2,'D20:I22'); % OCL
Requirement_Matrix{7} = xlsread('Case.xlsx',2,'D23:I25'); % OPS
Requirement_Matrix{8} = xlsread('Case.xlsx',2,'D26:I28'); % OAS
Requirement_Matrix{9} = xlsread('Case.xlsx',2,'D29:I31'); % LU

%% Transition matrix for techncial requirements 
TM_Requirement = xlsread('Case.xlsx',7,'B2:D4');

%% Changeability Cost Matrix. 
Change_CM = xlsread('Case.xlsx',8,'B3:G5'); 

%% Dayrates for each mission for each market state
Dayrates_Data = cell(1,N_Space_Market);
Dayrates_Data{1,1} = xlsread('Case.xlsx',5,'D10:E18'); % Low (-)
Dayrates_Data{1,2} = xlsread('Case.xlsx',5,'F10:G18'); % Low
Dayrates_Data{1,3} = xlsread('Case.xlsx',5,'H10:I18'); % Medium-low
Dayrates_Data{1,4} = xlsread('Case.xlsx',5,'J10:K18'); % Medium-high
Dayrates_Data{1,5} = xlsread('Case.xlsx',5,'L10:M18'); % High 
Dayrates_Data{1,6} = xlsread('Case.xlsx',5,'N10:O18'); % High (+)

% Difference between dayrates for short-term and long-term contracts 
ScalingFactor = xlsread('Case.xlsx',5,'D5:F5'); 

%% Create entire (full) design space by enumerating all possible ...
% ... combinations of the variable design state parameters. Some of ...
% ... these designs are not fesiable. 
% NB: [Space_Design_Var_full] = [ACC level, MC level, LWI level, ...
% ... PC level, MP level]
[Space_Design_Var_full] = allcomb(Set_ACC,Set_MC, Set_LWI,Set_ROV, Set_PC,Set_MP);
N_Space_Design_Var_full = length(Space_Design_Var_full);

%% Obtain information regarding all designs in "Space_Design_Var_full."
% NB: [Space_Design_Fix_full] = [Deck Area, Freeboard,Deadweight, GM]
[Space_Design_Fix_full, Lightweight] = Design_Information(Space_Design_Var_full,N_Space_Design_Var_full, N_Var_Design, Length, Beam,Depth, F_min);

%% Obtain compitability matrix 
V = [N_ACC, N_MC, N_LWI, N_ROV, N_PC, N_MP];
Design_Compatability_Matrix = cell(N_Var_Design);
counter = 4; 
for ii = 1: N_Var_Design
    for jj = 1:V(ii)
        Design_Compatability_Matrix{ii}(jj,:,1) = xlsread('Case.xlsx',1, strcat('C',num2str(counter),':E',num2str(counter)));
        Design_Compatability_Matrix{ii}(jj,:,2) = xlsread('Case.xlsx',1, strcat('F',num2str(counter),':H',num2str(counter)));
        Design_Compatability_Matrix{ii}(jj,:,3) = xlsread('Case.xlsx',1, strcat('I',num2str(counter),':K',num2str(counter)));
        Design_Compatability_Matrix{ii}(jj,:,4) = xlsread('Case.xlsx',1, strcat('L',num2str(counter),':N',num2str(counter)));
        Design_Compatability_Matrix{ii}(jj,:,5) = xlsread('Case.xlsx',1, strcat('O',num2str(counter),':Q',num2str(counter)));
        Design_Compatability_Matrix{ii}(jj,:,6) = xlsread('Case.xlsx',1, strcat('R',num2str(counter),':T',num2str(counter)));
        counter = counter + 1; 
    end
end
%% Create the total fesiable Design Space by deleating unfesiable designs ...
% ... in [Space_Design_Var_full] based in Design State Requirements ...
% ... (i.e. required deck area, GM and freeboard).  
% NB: [Space_Design] = [ACC level, MC level, LWI level, ...
% ... PC level, MP level]
[Space_Design_New] = Design_Compatibility(Design_Compatability_Matrix, Space_Design_Var_full,Space_Design_Fix_full, N_Var_Design, F_min, GM_min, DA_min);
N_Space_Design_New = length(Space_Design_New);  

%% Create the actual design space to analyse 
% It is assumed that one never will reduce the level of equipment on a
% vessel. Thus, only designs with equal or more equipment capability that
% the initial design (Start_Design) is analysed

[Space_Design] = Create_Space_Design(Space_Design_New,N_Space_Design_New,N_Var_Design,Start_Design); 
N_Space_Design = length(Space_Design);  

%% Initialize starting state (i.e. base design) 
state_idx_start = find(Space_Design(:,1) == Start_Design(1) & Space_Design(:,2) == Start_Design(2) & Space_Design(:,3) == Start_Design(3) & Space_Design(:,4) == Start_Design(4) & Space_Design(:,5) == Start_Design(5) & Space_Design(:,6) == Start_Design(6));

%% Creating Sets of Desigs, Missions, Markets, Strategy and Requirements 
Set_Design = 1:N_Space_Design; % Set of Design States
Set_Mission = 1:N_Space_Mission; % Set of Mission states
Set_Market = 1:N_Space_Market; % Set of Market states
Set_Strategy = 1:N_Strategy; % Sef of Strategy states
Set_Requirement = 1:N_Requirement; % Set of Requirement states

%% Create State Space by enumerating all combinations of ...
% ... design states, strategy states, market states and requirement states. 
% NB: [Space_State] = [Design nr., Strategy nr., Market nr., ...
% ... Requirement nr.] 
[Space_State] = allcomb(Set_Design, Set_Strategy, Set_Market, Set_Requirement);
N_Space_State = length(Space_State); % Number of possible states

%% Create Action Space by enumerating all ...
% ... combinations of design states and strategy states. 
% NB: [Space_Action] = (Design nr., Strategy nr.)
Space_Action = allcomb(Set_Design, Set_Strategy);
N_Space_Action = length(Space_Action);

%% Generate Switchign costs 
[switch_cost] = SwitchCost(Switch_CM,N_Space_Design, N_Var_Design, Space_Design,Change_CM,N_Design); 

%% Building Cost Estimation
FaktorCost = 8;
BuildCost = Lightweight*FaktorCost+switch_cost{1}(1,state_idx_start)*100/1000000;
SellingCost = [0*BuildCost, 0.1*BuildCost, 0.3*BuildCost, 0.5*BuildCost, 0.6*BuildCost,0.7*BuildCost];   
